from datetime import datetime
from unittest import TestCase

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from embedin.model.embedding_model import EmbeddingModel, Base
from embedin.repository.embedding_repository import EmbeddingRepository

engine = create_engine("sqlite:///:memory:", echo=True)
Session = sessionmaker(bind=engine)


class TestEmbeddingRepository(TestCase):
    def setUp(self):
        Base.metadata.create_all(engine)
        self.session = Session()
        self.repository = EmbeddingRepository(self.session)

        # Create some EmbeddingModel instances for testing
        self.embeddings = [
            EmbeddingModel(
                id="id1",
                collection_id="collection1",
                text="some text",
                embedding_data=[1.0, 2.0, 3.0],
                meta_data={"key1": "value1"},
                hash="hash1",
                created_at=datetime.now(),
            ),
            EmbeddingModel(
                id="id2",
                collection_id="collection1",
                text="some other text",
                embedding_data=[4.0, 5.0, 6.0],
                meta_data={"key2": "value2"},
                hash="hash2",
                created_at=datetime.now(),
            ),
        ]

    def tearDown(self):
        self.session.rollback()
        # Close the session and drop the in-memory database after testing
        self.session.close()
        Base.metadata.drop_all(engine)

    def test_add_rows_one_by_one(self):
        self.assertEqual(len(self.session.query(EmbeddingModel).all()), 0)
        self.repository._add_rows_one_by_one(self.embeddings)
        self.assertEqual(len(self.session.query(EmbeddingModel).all()), 2)

        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id1").first().text,
            "some text",
        )
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id2").first().text,
            "some other text",
        )

    def test__add_rows_one_by_one(self):
        self.repository.add_all(self.embeddings)
        # Test adding duplicate embeddings
        duplicate_embeddings = [
            EmbeddingModel(
                id="id3",
                collection_id="collection1",
                text="some text",
                embedding_data=[1.0, 2.0, 3.0],
                meta_data={"key1": "value1"},
                hash="hash1",
                created_at=datetime.now(),
            ),
            EmbeddingModel(
                id="id4",
                collection_id="collection1",
                text="some new text",
                embedding_data=[7.0, 8.0, 9.0],
                meta_data={"key3": "value3"},
                hash="hash4",
                created_at=datetime.now(),
            ),
        ]
        self.repository._add_rows_one_by_one(duplicate_embeddings)
        self.assertEqual(len(self.session.query(EmbeddingModel).all()), 3)
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id1").first().text,
            "some text",
        )
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id2").first().text,
            "some other text",
        )
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id4").first().text,
            "some new text",
        )

    def test_add_all(self):
        self.assertEqual(len(self.session.query(EmbeddingModel).all()), 0)
        self.repository.add_all(self.embeddings)
        self.assertEqual(len(self.session.query(EmbeddingModel).all()), 2)

        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id1").first().text,
            "some text",
        )
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id2").first().text,
            "some other text",
        )

    def test_add_all_with_duplicates(self):
        self.repository.add_all(self.embeddings)
        # Test adding duplicate embeddings
        duplicate_embeddings = [
            EmbeddingModel(
                id="id3",
                collection_id="collection1",
                text="some text",
                embedding_data=[1.0, 2.0, 3.0],
                meta_data={"key1": "value1"},
                hash="hash1",
                created_at=datetime.now(),
            ),
            EmbeddingModel(
                id="id4",
                collection_id="collection1",
                text="some new text",
                embedding_data=[7.0, 8.0, 9.0],
                meta_data={"key3": "value3"},
                hash="hash4",
                created_at=datetime.now(),
            ),
        ]
        self.repository.add_all(duplicate_embeddings)
        self.assertEqual(len(self.session.query(EmbeddingModel).all()), 3)
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id1").first().text,
            "some text",
        )
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id2").first().text,
            "some other text",
        )
        self.assertEqual(
            self.session.query(EmbeddingModel).filter_by(id="id4").first().text,
            "some new text",
        )

    def test_get_by_ids(self):
        self.repository.add_all(self.embeddings)
        ids = ["id1", "id2"]
        rows = self.repository.get_by_ids(ids)
        self.assertEqual(len(rows), 2)
        self.assertEqual(rows[0].id, "id1")
        self.assertEqual(rows[0].collection_id, "collection1")
        self.assertEqual(rows[0].text, "some text")
        self.assertEqual(rows[0].embedding_data, [1.0, 2.0, 3.0])
        self.assertEqual(rows[0].meta_data, {"key1": "value1"})
        self.assertEqual(rows[0].hash, "hash1")
        self.assertIsInstance(rows[0].created_at, datetime)
        self.assertEqual(rows[1].id, "id2")
        self.assertEqual(rows[1].collection_id, "collection1")
        self.assertEqual(rows[1].text, "some other text")
        self.assertEqual(rows[1].embedding_data, [4.0, 5.0, 6.0])
        self.assertEqual(rows[1].meta_data, {"key2": "value2"})
        self.assertEqual(rows[1].hash, "hash2")
        self.assertIsInstance(rows[1].created_at, datetime)

    def test_get_by_collection_id(self):
        self.repository.add_all(self.embeddings)
        collection_id = "collection1"
        rows = self.repository.get_by_collection_id(collection_id)
        self.assertEqual(len(rows), 2)
        self.assertEqual(rows[0].id, "id1")
        self.assertEqual(rows[0].collection_id, "collection1")
        self.assertEqual(rows[0].text, "some text")
        self.assertEqual(rows[0].embedding_data, [1.0, 2.0, 3.0])
        self.assertEqual(rows[0].meta_data, {"key1": "value1"})
        self.assertEqual(rows[0].hash, "hash1")
        self.assertIsInstance(rows[0].created_at, datetime)
        self.assertEqual(rows[1].id, "id2")
        self.assertEqual(rows[1].collection_id, "collection1")
        self.assertEqual(rows[1].text, "some other text")
        self.assertEqual(rows[1].embedding_data, [4.0, 5.0, 6.0])
        self.assertEqual(rows[1].meta_data, {"key2": "value2"})
        self.assertEqual(rows[1].hash, "hash2")
        self.assertIsInstance(rows[1].created_at, datetime)
